
type t = int
exception Bad_char

let max_char = 0x10FFFF
let max_char32 = Int32.of_int 0x10FFFF

let of_char c = ((Char.code c):t)
let of_int i =
  if (i < 0) || (i > max_char) then
    raise Bad_char
  else
    (i:t)
let to_int (c:t) =
  (c:int)
let of_int32 i =
  if (i < Int32.zero) || (i > max_char32) then
    raise Bad_char
  else
    of_int (Int32.to_int i)

let is_char (c':t) =
  let c = to_int c' in
    c = 0x09 || c = 0x0A || c = 0x0D ||
    c >= 0x20 && c <= 0xD7FF ||
    c >= 0xE000 && c <= 0xFFFD ||
    c >= 0x10000 && c <= 0x10FFFF
       
let is_space (c:t) =
  (match to_int c with
       0x20 | 0x09 | 0x0D | 0x0A -> true
     | _ -> false)

let is_digit (c':t) =
  let c = to_int c' in
    (c >= 48 && c <= 57) ||
    (c >= 1632 && c <= 1641) ||
    (c >= 1776 && c <= 1785) ||
    (c >= 2406 && c <= 2415) ||
    (c >= 2534 && c <= 2543) ||
    (c >= 2662 && c <= 2671) ||
    (c >= 2790 && c <= 2799) ||
    (c >= 2918 && c <= 2927) ||
    (c >= 3047 && c <= 3055) ||
    (c >= 3174 && c <= 3183) ||
    (c >= 3302 && c <= 3311) ||
    (c >= 3430 && c <= 3439) ||
    (c >= 3664 && c <= 3673) ||
    (c >= 3792 && c <= 3801) ||
    (c >= 3872 && c <= 3881)

let is_basechar (c':t) =
  let c = to_int c' in
    (c >= 65 && c <= 90) ||
    (c >= 97 && c <= 122) ||
    (c >= 192 && c <= 214) ||
    (c >= 216 && c <= 246) ||
    (c >= 248 && c <= 255) ||
    (c >= 256 && c <= 305) ||
    (c >= 308 && c <= 318) ||
    (c >= 321 && c <= 328) ||
    (c >= 330 && c <= 382) ||
    (c >= 384 && c <= 451) ||
    (c >= 461 && c <= 496) ||
    (c >= 500 && c <= 501) ||
    (c >= 506 && c <= 535) ||
    (c >= 592 && c <= 680) ||
    (c >= 699 && c <= 705) ||
    c = 902 ||
    (c >= 904 && c <= 906) ||
    c = 908 ||
    (c >= 910 && c <= 929) ||
    (c >= 931 && c <= 974) ||
    (c >= 976 && c <= 982) ||
    c = 986 ||
    c = 988 ||
    c = 990 ||
    c = 992 ||
    (c >= 994 && c <= 1011) ||
    (c >= 1025 && c <= 1036) ||
    (c >= 1038 && c <= 1103) ||
    (c >= 1105 && c <= 1116) ||
    (c >= 1118 && c <= 1153) ||
    (c >= 1168 && c <= 1220) ||
    (c >= 1223 && c <= 1224) ||
    (c >= 1227 && c <= 1228) ||
    (c >= 1232 && c <= 1259) ||
    (c >= 1262 && c <= 1269) ||
    (c >= 1272 && c <= 1273) ||
    (c >= 1329 && c <= 1366) ||
    c = 1369 ||
    (c >= 1377 && c <= 1414) ||
    (c >= 1488 && c <= 1514) ||
    (c >= 1520 && c <= 1522) ||
    (c >= 1569 && c <= 1594) ||
    (c >= 1601 && c <= 1610) ||
    (c >= 1649 && c <= 1719) ||
    (c >= 1722 && c <= 1726) ||
    (c >= 1728 && c <= 1742) ||
    (c >= 1744 && c <= 1747) ||
    c = 1749 ||
    (c >= 1765 && c <= 1766) ||
    (c >= 2309 && c <= 2361) ||
    c = 2365 ||
    (c >= 2392 && c <= 2401) ||
    (c >= 2437 && c <= 2444) ||
    (c >= 2447 && c <= 2448) ||
    (c >= 2451 && c <= 2472) ||
    (c >= 2474 && c <= 2480) ||
    c = 2482 ||
    (c >= 2486 && c <= 2489) ||
    (c >= 2524 && c <= 2525) ||
    (c >= 2527 && c <= 2529) ||
    (c >= 2544 && c <= 2545) ||
    (c >= 2565 && c <= 2570) ||
    (c >= 2575 && c <= 2576) ||
    (c >= 2579 && c <= 2600) ||
    (c >= 2602 && c <= 2608) ||
    (c >= 2610 && c <= 2611) ||
    (c >= 2613 && c <= 2614) ||
    (c >= 2616 && c <= 2617) ||
    (c >= 2649 && c <= 2652) ||
    c = 2654 ||
    (c >= 2674 && c <= 2676) ||
    (c >= 2693 && c <= 2699) ||
    c = 2701 ||
    (c >= 2703 && c <= 2705) ||
    (c >= 2707 && c <= 2728) ||
    (c >= 2730 && c <= 2736) ||
    (c >= 2738 && c <= 2739) ||
    (c >= 2741 && c <= 2745) ||
    c = 2749 ||
    c = 2784 ||
    (c >= 2821 && c <= 2828) ||
    (c >= 2831 && c <= 2832) ||
    (c >= 2835 && c <= 2856) ||
    (c >= 2858 && c <= 2864) ||
    (c >= 2866 && c <= 2867) ||
    (c >= 2870 && c <= 2873) ||
    c = 2877 ||
    (c >= 2908 && c <= 2909) ||
    (c >= 2911 && c <= 2913) ||
    (c >= 2949 && c <= 2954) ||
    (c >= 2958 && c <= 2960) ||
    (c >= 2962 && c <= 2965) ||
    (c >= 2969 && c <= 2970) ||
    c = 2972 ||
    (c >= 2974 && c <= 2975) ||
    (c >= 2979 && c <= 2980) ||
    (c >= 2984 && c <= 2986) ||
    (c >= 2990 && c <= 2997) ||
    (c >= 2999 && c <= 3001) ||
    (c >= 3077 && c <= 3084) ||
    (c >= 3086 && c <= 3088) ||
    (c >= 3090 && c <= 3112) ||
    (c >= 3114 && c <= 3123) ||
    (c >= 3125 && c <= 3129) ||
    (c >= 3168 && c <= 3169) ||
    (c >= 3205 && c <= 3212) ||
    (c >= 3214 && c <= 3216) ||
    (c >= 3218 && c <= 3240) ||
    (c >= 3242 && c <= 3251) ||
    (c >= 3253 && c <= 3257) ||
    c = 3294 ||
    (c >= 3296 && c <= 3297) ||
    (c >= 3333 && c <= 3340) ||
    (c >= 3342 && c <= 3344) ||
    (c >= 3346 && c <= 3368) ||
    (c >= 3370 && c <= 3385) ||
    (c >= 3424 && c <= 3425) ||
    (c >= 3585 && c <= 3630) ||
    c = 3632 ||
    (c >= 3634 && c <= 3635) ||
    (c >= 3648 && c <= 3653) ||
    (c >= 3713 && c <= 3714) ||
    c = 3716 ||
    (c >= 3719 && c <= 3720) ||
    c = 3722 ||
    c = 3725 ||
    (c >= 3732 && c <= 3735) ||
    (c >= 3737 && c <= 3743) ||
    (c >= 3745 && c <= 3747) ||
    c = 3749 ||
    c = 3751 ||
    (c >= 3754 && c <= 3755) ||
    (c >= 3757 && c <= 3758) ||
    c = 3760 ||
    (c >= 3762 && c <= 3763) ||
    c = 3773 ||
    (c >= 3776 && c <= 3780) ||
    (c >= 3904 && c <= 3911) ||
    (c >= 3913 && c <= 3945) ||
    (c >= 4256 && c <= 4293) ||
    (c >= 4304 && c <= 4342) ||
    c = 4352 ||
    (c >= 4354 && c <= 4355) ||
    (c >= 4357 && c <= 4359) ||
    c = 4361 ||
    (c >= 4363 && c <= 4364) ||
    (c >= 4366 && c <= 4370) ||
    c = 4412 ||
    c = 4414 ||
    c = 4416 ||
    c = 4428 ||
    c = 4430 ||
    c = 4432 ||
    (c >= 4436 && c <= 4437) ||
    c = 4441 ||
    (c >= 4447 && c <= 4449) ||
    c = 4451 ||
    c = 4453 ||
    c = 4455 ||
    c = 4457 ||
    (c >= 4461 && c <= 4462) ||
    (c >= 4466 && c <= 4467) ||
    c = 4469 ||
    c = 4510 ||
    c = 4520 ||
    c = 4523 ||
    (c >= 4526 && c <= 4527) ||
    (c >= 4535 && c <= 4536) ||
    c = 4538 ||
    (c >= 4540 && c <= 4546) ||
    c = 4587 ||
    c = 4592 ||
    c = 4601 ||
    (c >= 7680 && c <= 7835) ||
    (c >= 7840 && c <= 7929) ||
    (c >= 7936 && c <= 7957) ||
    (c >= 7960 && c <= 7965) ||
    (c >= 7968 && c <= 8005) ||
    (c >= 8008 && c <= 8013) ||
    (c >= 8016 && c <= 8023) ||
    c = 8025 ||
    c = 8027 ||
    c = 8029 ||
    (c >= 8031 && c <= 8061) ||
    (c >= 8064 && c <= 8116) ||
    (c >= 8118 && c <= 8124) ||
    c = 8126 ||
    (c >= 8130 && c <= 8132) ||
    (c >= 8134 && c <= 8140) ||
    (c >= 8144 && c <= 8147) ||
    (c >= 8150 && c <= 8155) ||
    (c >= 8160 && c <= 8172) ||
    (c >= 8178 && c <= 8180) ||
    (c >= 8182 && c <= 8188) ||
    c = 8486 ||
    (c >= 8490 && c <= 8491) ||
    c = 8494 ||
    (c >= 8576 && c <= 8578) ||
    (c >= 12353 && c <= 12436) ||
    (c >= 12449 && c <= 12538) ||
    (c >= 12549 && c <= 12588) ||
    (c >= 44032 && c <= 55203)

let is_ideographic (c':t) =
  let c = to_int c' in
    (c >= 19968 && c <= 40869) ||
    c = 12295 ||
    (c >= 12321 && c <= 12329)
  
let is_letter (c:t) =
  is_basechar c || is_ideographic c

let is_combiningchar (c':t) =
  let c = to_int c' in
    (c >= 768 && c <= 837) ||
    (c >= 864 && c <= 865) ||
    (c >= 1155 && c <= 1158) ||
    (c >= 1425 && c <= 1441) ||
    (c >= 1443 && c <= 1465) ||
    (c >= 1467 && c <= 1469) ||
    c = 1471 ||
    (c >= 1473 && c <= 1474) ||
    c = 1476 ||
    (c >= 1611 && c <= 1618) ||
    c = 1648 ||
    (c >= 1750 && c <= 1756) ||
    (c >= 1757 && c <= 1759) ||
    (c >= 1760 && c <= 1764) ||
    (c >= 1767 && c <= 1768) ||
    (c >= 1770 && c <= 1773) ||
    (c >= 2305 && c <= 2307) ||
    c = 2364 ||
    (c >= 2366 && c <= 2380) ||
    c = 2381 ||
    (c >= 2385 && c <= 2388) ||
    (c >= 2402 && c <= 2403) ||
    (c >= 2433 && c <= 2435) ||
    c = 2492 ||
    c = 2494 ||
    c = 2495 ||
    (c >= 2496 && c <= 2500) ||
    (c >= 2503 && c <= 2504) ||
    (c >= 2507 && c <= 2509) ||
    c = 2519 ||
    (c >= 2530 && c <= 2531) ||
    c = 2562 ||
    c = 2620 ||
    c = 2622 ||
    c = 2623 ||
    (c >= 2624 && c <= 2626) ||
    (c >= 2631 && c <= 2632) ||
    (c >= 2635 && c <= 2637) ||
    (c >= 2672 && c <= 2673) ||
    (c >= 2689 && c <= 2691) ||
    c = 2748 ||
    (c >= 2750 && c <= 2757) ||
    (c >= 2759 && c <= 2761) ||
    (c >= 2763 && c <= 2765) ||
    (c >= 2817 && c <= 2819) ||
    c = 2876 ||
    (c >= 2878 && c <= 2883) ||
    (c >= 2887 && c <= 2888) ||
    (c >= 2891 && c <= 2893) ||
    (c >= 2902 && c <= 2903) ||
    (c >= 2946 && c <= 2947) ||
    (c >= 3006 && c <= 3010) ||
    (c >= 3014 && c <= 3016) ||
    (c >= 3018 && c <= 3021) ||
    c = 3031 ||
    (c >= 3073 && c <= 3075) ||
    (c >= 3134 && c <= 3140) ||
    (c >= 3142 && c <= 3144) ||
    (c >= 3146 && c <= 3149) ||
    (c >= 3157 && c <= 3158) ||
    (c >= 3202 && c <= 3203) ||
    (c >= 3262 && c <= 3268) ||
    (c >= 3270 && c <= 3272) ||
    (c >= 3274 && c <= 3277) ||
    (c >= 3285 && c <= 3286) ||
    (c >= 3330 && c <= 3331) ||
    (c >= 3390 && c <= 3395) ||
    (c >= 3398 && c <= 3400) ||
    (c >= 3402 && c <= 3405) ||
    c = 3415 ||
    c = 3633 ||
    (c >= 3636 && c <= 3642) ||
    (c >= 3655 && c <= 3662) ||
    c = 3761 ||
    (c >= 3764 && c <= 3769) ||
    (c >= 3771 && c <= 3772) ||
    (c >= 3784 && c <= 3789) ||
    (c >= 3864 && c <= 3865) ||
    c = 3893 ||
    c = 3895 ||
    c = 3897 ||
    c = 3902 ||
    c = 3903 ||
    (c >= 3953 && c <= 3972) ||
    (c >= 3974 && c <= 3979) ||
    (c >= 3984 && c <= 3989) ||
    c = 3991 ||
    (c >= 3993 && c <= 4013) ||
    (c >= 4017 && c <= 4023) ||
    c = 4025 ||
    (c >= 8400 && c <= 8412) ||
    c = 8417 ||
    (c >= 12330 && c <= 12335) ||
    c = 12441 ||
    c = 12442

let is_extender (c':t) =
  let c = to_int c' in
    c = 183 ||
    c = 720 ||
    c = 721 ||
    c = 903 ||
    c = 1600 ||
    c = 3654 ||
    c = 3782 ||
    c = 12293 ||
    (c >= 12337 && c <= 12341) ||
    (c >= 12445 && c <= 12446) ||
    (c >= 12540 && c <= 12542)

let is_namechar (c:t) =
  is_letter c || is_digit c || c = of_char '.' || c = of_char '-' ||
  c = of_char '_' || c = of_char ':' || is_combiningchar c || is_extender c

let is_ncnamechar (c:t) =
  is_letter c || is_digit c || c = of_char '.' || c = of_char '-' ||
  c = of_char '_' || is_combiningchar c || is_extender c

